@@ -56,8 +56,11 @@ from pkg_resources import parse_version
 
 # python2/3 compatibility
 try:
+    from urllib.error import HTTPError, URLError
     from urllib.parse import urlparse
+    from urllib.request import Request, urlopen
 except ImportError:
+    from urllib2 import HTTPError, Request, URLError, urlopen
     from urlparse import urlparse
 
 import bloom
@@ -77,6 +80,7 @@ from bloom.github import auth_header_from_basic_auth
 from bloom.github import auth_header_from_oauth_token
 from bloom.github import Github
 from bloom.github import GithubException
+from bloom.github import GitHubAuthException
 
 from bloom.logging import debug
 from bloom.logging import error
@@ -295,6 +299,27 @@ def validate_github_url(url, url_type):
     return True
 
 
+def infer_release_repo_from_env(repository):
+    """
+    Generate a release repo url from a hint variable.
+
+    If the environment var BLOOM_RELEASE_REPO_BASE exists, and
+    BLOOM_RELEASE_REPO_BASE + repository + '-release.git' exists online, then
+    this function will return the newly composed url
+    """
+    base = os.environ.get('BLOOM_RELEASE_REPO_BASE', None)
+    if base is None:
+        return None
+    url = base + repository + '-release.git'
+    try:
+        urlopen(Request(url))
+    except URLError:
+        return None
+    except HTTPError:
+        return None
+    return url
+
+
 def get_repo_uri(repository, distro):
     url = None
     # Fetch the distro file
@@ -308,6 +333,8 @@ def get_repo_uri(repository, distro):
         matches = difflib.get_close_matches(repository, distribution_file.repositories)
         if matches:
             info(fmt("@{yf}Did you mean one of these: '" + "', '".join([m for m in matches]) + "'?"))
+    if url is None:
+        url = infer_release_repo_from_env(repository)
     if url is None:
         info("Could not determine release repository url for repository '{0}' of distro '{1}'"
              .format(repository, distro))
@@ -320,7 +347,7 @@ def get_repo_uri(repository, distro):
         default_distro, default_release = get_most_recent('release', repository)
         default_release_repo_url = default_release.url if default_release else "press enter to abort"
         if default_distro is not None:
-            warning("A previous distribution, '{0}', released this repository.".format(default_distro))
+            warning("A different distribution, '{0}', released this repository.".format(default_distro))
         else:
             warning("No reasonable default release repository url could be determined from previous releases.")
         while True:
@@ -638,6 +665,19 @@ _gh = None
 
 
 def get_github_interface(quiet=False):
+    def mfa_prompt(oauth_config_path, username):
+        """Explain how to create a token for users with Multi-Factor Authentication configured."""
+        warning("Receiving 401 when trying to create an oauth token can be caused by the user "
+                "having two-factor authentication enabled.")
+        warning("If 2FA is enabled, the user will have to create an oauth token manually.")
+        warning("A token can be created at https://github.com/settings/applications")
+        warning("The resulting token can be placed in the '{oauth_config_path}' file as such:"
+                .format(**locals()))
+        info("")
+        warning('{{"github_user": "{username}", "oauth_token": "TOKEN_GOES_HERE"}}'
+                .format(**locals()))
+        info("")
+
     global _gh
     if _gh is not None:
         return _gh
@@ -686,20 +726,14 @@ def get_github_interface(quiet=False):
                 f.write(json.dumps(config))
             info("The token '{token}' was created and stored in the bloom config file: '{oauth_config_path}'"
                  .format(**locals()))
+        except GitHubAuthException as exc:
+            error("{0}".format(exc))
+            mfa_prompt(oauth_config_path, username)
         except GithubException as exc:
             error("{0}".format(exc))
             info("")
-            if '{0}'.format(exc.resp.status) in ['401']:
-                warning("Receiving 401 when trying to create an oauth token can be caused by the user "
-                        "having two-factor authentication enabled.")
-                warning("If 2FA is enabled, the user will have to create an oauth token manually.")
-                warning("A token can be created at https://github.com/settings/applications")
-                warning("The resulting token can be placed in the '{oauth_config_path}' file as such:"
-                        .format(**locals()))
-                info("")
-                warning('{{"github_user": "{username}", "oauth_token": "TOKEN_GOES_HERE"}}'
-                        .format(**locals()))
-                info("")
+            if hasattr(exc, 'resp') and '{0}'.format(exc.resp.status) in ['401']:
+                mfa_prompt(oauth_config_path, username)
             warning("This sometimes fails when the username or password are incorrect, try again?")
             if not maybe_continue():
                 return None
@@ -1258,7 +1292,7 @@ def get_argument_parser():
     add('repository', help="repository to run bloom on")
     add('--list-tracks', '-l', action='store_true', default=False,
         help="list available tracks for repository")
-    add('--track', '-t', required=True, help="track to run")
+    add('--track', '-t', required=False, help="track to run; defaults to rosdistro name")
     add('--non-interactive', '-y', action='store_true', default=False)
     add('--ros-distro', '--rosdistro', '-r', required=True,
         help="determines the ROS distro file used")
@@ -1285,6 +1319,8 @@ def main(sysargs=None):
     parser = get_argument_parser()
     parser = add_global_arguments(parser)
     args = parser.parse_args(sysargs)
+    if args.track is None:
+        args.track = args.ros_distro
     handle_global_arguments(args)
 
     if args.list_tracks:
